/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE support@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * This file is a test script.
 *
 * @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgtestmacro.h"
#include "skgimportexportmanager.h"
#include "skgbankincludes.h"

/**
 * The main function of the unit test
 * @param argc the number of arguments
 * @param argv the list of arguments
 */
int main(int argc, char** argv)
{
    Q_UNUSED(argc);
    Q_UNUSED(argv);

    // Init test
    SKGINITTEST(true);

    {
        // Test import KMY
        SKGDocumentBank document1;
        SKGTESTERROR("document1.initialize()", document1.initialize(), true);
        SKGError err;
        {
            // Scope of the transaction
            SKGBEGINTRANSACTION(document1, "IMPORT_KMY", err);
            SKGImportExportManager impmissing(&document1, QUrl("missingfile.kmy"));
            SKGTESTERROR("imp1.importFile", impmissing.importFile(), false);

            SKGImportExportManager imp1(&document1, QUrl::fromLocalFile(SKGTest::getTestPath("IN") % "/skgtestimportkmy/rapprochement-incorrect.kmy"));
            SKGTESTERROR("KMY.importFile", imp1.importFile(), true);
        }

        {
            SKGAccountObject account(&document1);
            SKGTESTERROR("KMY.setName", account.setName("CCP"), true);
            SKGTESTERROR("KMY.load", account.load(), true);
            SKGTEST("KMY:getCurrentAmount", SKGServices::doubleToString(account.getCurrentAmount()), "750");
        }

        {
            SKGAccountObject account(&document1);
            SKGTESTERROR("KMY.setName", account.setName("CEL Donald"), true);
            SKGTESTERROR("KMY.load", account.load(), true);
            SKGTEST("KMY:getCurrentAmount", SKGServices::doubleToString(account.getCurrentAmount()), "150");
        }

        {
            SKGAccountObject account(&document1);
            SKGTESTERROR("KMY.setName", account.setName("PEEs"), true);
            SKGTESTERROR("KMY.load", account.load(), true);
            SKGTEST("KMY:getCurrentAmount", SKGServices::doubleToString(account.getCurrentAmount()), "640");
        }

        {
            // Scope of the transaction
            SKGImportExportManager imp1(&document1, QUrl::fromLocalFile(SKGTest::getTestPath("OUT") % "/skgtestimportkmy1/rapprochement-incorrect.kmy"));
            SKGTESTERROR("KMY.exportFile", imp1.exportFile(), true);
        }
    }

    {
        // Test import KMY
        SKGDocumentBank document1;
        SKGTESTERROR("document1.initialize()", document1.initialize(), true);
        SKGError err;
        {
            // Scope of the transaction
            SKGBEGINTRANSACTION(document1, "IMPORT_KMY", err);

            SKGImportExportManager imp1(&document1, QUrl::fromLocalFile(SKGTest::getTestPath("IN") % "/skgtestimportkmy/mytest.kmy"));
            SKGTESTERROR("KMY.importFile", imp1.importFile(), true);
        }

        int nb = 0;
        SKGTESTERROR("KMY:getNbObjects(recurrentoperation)", document1.getNbObjects("recurrentoperation", "", nb), true);
        SKGTEST("KMY:getNbObjects(recurrentoperation)", nb, 8);
    }

    {
        // Test import KMY
        SKGDocumentBank document1;
        SKGTESTERROR("document1.initialize()", document1.initialize(), true);
        SKGError err;
        {
            // Scope of the transaction
            SKGBEGINTRANSACTION(document1, "IMPORT_KMY", err);

            SKGImportExportManager imp1(&document1, QUrl::fromLocalFile(SKGTest::getTestPath("IN") % "/skgtestimportkmy/action.kmy"));
            SKGTESTERROR("KMY.importFile", imp1.importFile(), true);
        }

        {
            SKGAccountObject account(&document1);
            SKGTESTERROR("KMY.setName", account.setName("courant"), true);
            SKGTESTERROR("KMY.load", account.load(), true);
            SKGTEST("KMY:getCurrentAmount", SKGServices::doubleToString(account.getCurrentAmount()), "-600");
        }

        {
            SKGAccountObject account(&document1);
            SKGTESTERROR("KMY.setName", account.setName("actions"), true);
            SKGTESTERROR("KMY.load", account.load(), true);
            SKGTEST("KMY:getCurrentAmount", SKGServices::doubleToString(account.getCurrentAmount()), "100");
        }

        // test multi import
        {
            // Scope of the transaction
            SKGBEGINTRANSACTION(document1, "IMPORT_KMY", err);

            SKGImportExportManager imp1(&document1, QUrl::fromLocalFile(SKGTest::getTestPath("IN") % "/skgtestimportkmy/action.kmy"));
            SKGTESTERROR("KMY.importFile", imp1.importFile(), true);
        }
    }

    {
        // Test import KMY
        SKGDocumentBank document1;
        SKGTESTERROR("document1.initialize()", document1.initialize(), true);
        SKGError err;
        {
            // Scope of the transaction
            SKGBEGINTRANSACTION(document1, "IMPORT_KMY", err);

            SKGImportExportManager imp1(&document1, QUrl::fromLocalFile(SKGTest::getTestPath("IN") % "/skgtestimportkmy/252869.kmy"));
            SKGTESTERROR("KMY.importFile", imp1.importFile(), true);
        }

        int nb = 0;
        SKGTESTERROR("KMY:getNbObjects(recurrentoperation)", document1.getNbObjects("bank", "t_name='Test Bank'", nb), true);
        SKGTEST("KMY:getNbObjects(recurrentoperation)", nb, 1);
    }
    // End test
    SKGENDTEST();
}
