/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE support@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * This file is a test script.
 *
 * @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgtestmacro.h"
#include "skgimportexportmanager.h"
#include "skgbankincludes.h"

/**
 * The main function of the unit test
 * @param argc the number of arguments
 * @param argv the list of arguments
 */
int main(int argc, char** argv)
{
    Q_UNUSED(argc);
    Q_UNUSED(argv);

    // Init test
    SKGINITTEST(true);

    {
        // Test import GSK
        SKGDocumentBank document1;
        SKGTESTERROR("document1.initialize()", document1.initialize(), true);
        SKGError err;
        {
            // Scope of the transaction
            SKGBEGINTRANSACTION(document1, "IMPORT_MMB", err);
            SKGImportExportManager impmissing(&document1, QUrl("missingfile.mmb"));
            SKGTESTERROR("imp1.importFile", impmissing.importFile(), false);

            SKGImportExportManager imp1(&document1, QUrl::fromLocalFile(SKGTest::getTestPath("IN") % "/skgtestimportmmb/test.mmb"));
            SKGTESTERROR("MMB.importFile", imp1.importFile(), true);
        }

        {
            SKGAccountObject account(&document1);
            SKGTESTERROR("MMB.setName", account.setName("COURANT"), true);
            SKGTESTERROR("MMB.load", account.load(), true);
            SKGTEST("GSB:getCurrentAmount", SKGServices::doubleToString(account.getCurrentAmount()), "-144.05");
        }
        {
            SKGAccountObject account(&document1);
            SKGTESTERROR("MMB.setName", account.setName("CPT2"), true);
            SKGTESTERROR("MMB.load", account.load(), true);
            SKGTEST("GSB:getCurrentAmount", SKGServices::doubleToString(account.getCurrentAmount()), "40");
        }
        {
            SKGAccountObject account(&document1);
            SKGTESTERROR("MMB.setName", account.setName("NOTFAVORITE"), true);
            SKGTESTERROR("MMB.load", account.load(), true);
            SKGTEST("GSB:getCurrentAmount", SKGServices::doubleToString(account.getCurrentAmount()), "150");
        }
        {
            SKGAccountObject account(&document1);
            SKGTESTERROR("MMB.setName", account.setName("LEP"), true);
            SKGTESTERROR("MMB.load", account.load(), true);
            SKGTEST("GSB:getCurrentAmount", SKGServices::doubleToString(account.getCurrentAmount()), "6000");
        }
    }

    {
        // Test import GSK
        SKGDocumentBank document1;
        SKGTESTERROR("document1.initialize()", document1.initialize(), true);
        SKGError err;
        {
            // Scope of the transaction
            SKGBEGINTRANSACTION(document1, "IMPORT_MMB", err);

            SKGImportExportManager imp1(&document1, QUrl::fromLocalFile(SKGTest::getTestPath("IN") % "/skgtestimportmmb/jd.mmb"));
            SKGTESTERROR("MMB.importFile", imp1.importFile(), true);
        }

        {
            SKGAccountObject account(&document1);
            SKGTESTERROR("MMB.setName", account.setName("credit"), true);
            SKGTESTERROR("MMB.load", account.load(), true);
            SKGTEST("GSB:getCurrentAmount", SKGServices::doubleToString(account.getCurrentAmount()), "-974.04");
        }
    }
    // End test
    SKGENDTEST();
}
